@@ -1,6 +1,6 @@
 /* This testcase is part of GDB, the GNU debugger.
 
-   Copyright 2001, 2004, 2007-2012 Free Software Foundation, Inc.
+   Copyright 2001-2013 Free Software Foundation, Inc.
 
    Contributed by Red Hat, originally written by Jim Blandy.
 
@@ -123,10 +123,6 @@ extern void malloc_stub (void);
 
 int main ()
 {
-#ifdef usestubs
-  set_debug_traps();
-  breakpoint();
-#endif
 
   malloc_stub ();
 
